# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

# AG 2021: in pypi/comment_parser this module was only for parsing JavaScript.
# But in fact, it coped better with parsing C than old regex-based c_parser.py.
# So for now it is the default parsed for all C-like languages.
#
# The most evident difference between JS and C in that single quotes can
# contain a string in JS, but in C and C++ they are only for single chars.
# However, for the task, individual characters within single quotes are
# hardly different from single-letter strings.
#
# The parser for Go now also relies on the following code. In fact, the only
# key differnce in Go parsed was `backquoted strings` support

from enum import IntEnum, auto
from typing import Iterable

import commie.x01_errors
from commie import x01_common
from commie.x01_common import Comment, Span


def iter_comments_c(source: str) -> Iterable[Comment]:
	return _iter_comments_universal(source, "\"'")


def iter_comments_go(source: str) -> Iterable[Comment]:
	return _iter_comments_universal(source, "\"'`")


def _iter_comments_universal(source: str, string_quote_chars: str) -> Iterable[Comment]:
	class State(IntEnum):

		DEFAULT = auto()
		IN_SINGLE_LINE_COMMENT = auto()
		IN_MULTI_LINE_COMMENT = auto()
		IN_MULTI_LINE_COMMENT_AFTER_ASTERISK = auto()
		IN_STRING = auto()
		IN_STRING_AFTER_BACKSLASH = auto()
		FOUND_SLASH = auto()

	state: State = State.DEFAULT

	comment_start_pos = None

	quote = None
	position = 0

	for position, char in enumerate(source):

		if state == State.DEFAULT:
			# waiting for comment start character or beginning of a string
			if char == '/':
				state = State.FOUND_SLASH
			elif char in string_quote_chars:
				quote = char
				state = State.IN_STRING
		elif state == State.FOUND_SLASH:
			# found comment start character, classify next character and
			# determine if single or multi-line comment.
			if char == '/':
				state = State.IN_SINGLE_LINE_COMMENT
				comment_start_pos = position - 1  # we are at second char of "//"
			elif char == '*':
				state = State.IN_MULTI_LINE_COMMENT
				comment_start_pos = position - 1  # we are at second char of "/*"
			else:
				state = State.DEFAULT
		elif state == State.IN_SINGLE_LINE_COMMENT:
			# in single-line comment, reading characters until EOL
			if char == '\n':
				yield x01_common.Comment(
					source,
					text_span=Span(comment_start_pos + 2, position),
					code_span=Span(comment_start_pos, position),
					multiline=False)
				state = State.DEFAULT
		elif state == State.IN_MULTI_LINE_COMMENT:
			# in multi-line comment, add characters until '*' is encountered
			if char == '*':
				state = State.IN_MULTI_LINE_COMMENT_AFTER_ASTERISK
		elif state == State.IN_MULTI_LINE_COMMENT_AFTER_ASTERISK:
			# In multi-line comment with asterisk found. Determine if
			# comment is ending.
			if char == '/':
				yield Comment(
					source,
					text_span=Span(comment_start_pos + 2, position - 1),
					code_span=Span(comment_start_pos, position + 1),
					multiline=True)
				state = State.DEFAULT
			else:
				# care for multiple '*' in a row
				if char != '*':
					state = State.IN_MULTI_LINE_COMMENT
		elif state == State.IN_STRING:
			# in string literal, expect literal end or escape character.
			if char == quote:
				state = State.DEFAULT
			elif char == '\\':
				state = State.IN_STRING_AFTER_BACKSLASH
		elif state == State.IN_STRING_AFTER_BACKSLASH:
			state = State.IN_STRING

	# end of file
	if state in (State.IN_MULTI_LINE_COMMENT, State.IN_MULTI_LINE_COMMENT_AFTER_ASTERISK):
		raise commie.x01_errors.UnterminatedCommentError()

	if state == State.IN_SINGLE_LINE_COMMENT:
		yield x01_common.Comment(
			source,
			text_span=Span(comment_start_pos + 2, position + 1),
			code_span=Span(comment_start_pos, position + 1),
			multiline=False)



from .parsers import *
from .x01_common import Comment, Span
from .x01_errors import *
from .x02_detector import iter_comments_str, iter_comments_file, iter_comments
from .x03_glue import group_singleline_comments



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-License-Identifier: BSD-3-Clause

import re

from commie.x01_common import Comment, Span


def matchGroupToComment(match: re.Match, groupName: str, multiline: bool) -> Comment:
	fullSpan = match.span()

	fullText = match.group(0)
	innerText = match.group(groupName)
	textStart = fullText.index(innerText)
	assert textStart >= 0

	return Comment(
		match.string,
		text_span=Span(fullSpan[0] + textStart, fullSpan[0] + textStart + len(innerText)),
		code_span=Span(fullSpan[0], fullSpan[1]),
		multiline=multiline)



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

import io
import tokenize
from typing import NamedTuple, Iterable

from commie.x01_common import Comment, Span


class PosToken(NamedTuple):
	tokenType: int
	text: str
	start: int
	end: int


def postokenize(infile: io.BytesIO) -> Iterable[PosToken]:
	# based on https://stackoverflow.com/a/62761208 (CC BY-SA 4.0)

	# Used to track starting position of each line.
	# Note that tokenize starts line numbers at 1 and column numbers at 0
	offsets = [0]

	def wrapped_readline():
		# Function used to wrap calls to infile.readline(); stores current
		# stream position at the beginning of each line.
		offsets.append(infile.tell())
		return infile.readline()

	# For each returned token, substitute type with exact_type and
	# add token boundaries as stream positions
	for t in tokenize.tokenize(wrapped_readline):
		startline, startcol = t.start
		endline, endcol = t.end
		yield PosToken(t.exact_type, t.string,
					   offsets[startline] + startcol,
					   offsets[endline] + endcol)


def extract_comments(code: str) -> Iterable[Comment]:
	"""Extracts a list of comments from the given Python script.
	Comments are identified using the tokenize module. Does not include function,
	class, or module docstrings. All comments are single line comments.
	Args:
	  code: String containing code to extract comments from.
	Returns:
	  Python list of common.Comment in the order that they appear in the code.
	Raises:
	  tokenize.TokenError
	"""

	for token in postokenize(io.BytesIO(code.encode())):
		if token.tokenType == tokenize.COMMENT:
			yield Comment(
				code,
				text_span=Span(token.start + 1, token.end),
				code_span=Span(token.start, token.end),
				multiline=False
			)


if __name__ == "__main__":
	def experiment():
		from pathlib import Path
		extract_comments(Path(__file__).read_text())


	experiment()



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import List

from commie import iter_comments_sass
from commie.x01_common import Comment


def commentsToList(code: str) -> List[Comment]:
	return list(iter_comments_sass(code))


class SassParserTest(unittest.TestCase):

	def testEmptyString(self):
		code = ""
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testSimpleMain(self):
		code = ".cssClass { /* i am\n a comment! */ }"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "/* i am\n a comment! */")
		self.assertEqual(comments[0].text, " i am\n a comment! ")
		self.assertEqual(comments[0].multiline, True)

	def testIncLeft(self):
		code = ".cssClass { /* /* i am a comment! */ }"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "/* /* i am a comment! */")
		self.assertEqual(comments[0].text, " /* i am a comment! ")
		self.assertEqual(comments[0].multiline, True)

	def testIncRight(self):
		code = ".cssClass { /* i am a comment! */ */ }"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "/* i am a comment! */")
		self.assertEqual(comments[0].text, " i am a comment! ")
		self.assertEqual(comments[0].multiline, True)

	def testThreeComments(self):
		code = """
    		/* Welcome to Compass.
			 * In this file you should write your main styles. (or centralize your imports)
			 * Import this file using the following HTML or equivalent:
			 * <link href="/stylesheets/screen.css" media="screen, projection" rel="stylesheet" type="text/css" /> */
			
			@import "compass/reset";
			
			@import "compass/reset";
			@import "compass/css3";
			@import "compass/layout";
			@import "compass/utilities";
			@import "compass/utilities/general/clearfix";
			@import "compass/css3/pie";
			@import "compass/css3/transform";
			
			
			//Uses Sass' color modifying functions to adjust colors
			
			$color1: #a9c4e5;
			$color2: darken($color1, 40);
			$fontcolor1: darken($color1, 80);
			$fontcolor2: lighten($color1, 50);
			$border-radius: 4px;
			
			
			@import "basics";
    
    	"""
		comments = commentsToList(code)

		self.assertEqual(len(comments), 2)

		self.assertEqual(len(comments[0].code.splitlines()), 4)
		self.assertEqual(len(comments[1].code.splitlines()), 1)



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import NamedTuple, Optional


class Span(NamedTuple):
	start: int
	end: int

	def extract(self, text: str):
		return text[self.start:self.end]


class Comment:
	"""Represents comments found in a source code string."""

	def __init__(self, source: str, code_span: Span, text_span: Span, multiline: bool):

		self.source = source
		self.code_span: Span = code_span
		self.text_span: Span = text_span
		self.multiline = multiline

		self._text: Optional[str] = None
		self._code: Optional[str] = None

	@property
	def text(self) -> str:
		if self._text is None:
			self._text = self.text_span.extract(self.source)
		return self._text

	@property
	def code(self) -> str:
		if self._code is None:
			self._code = self.code_span.extract(self.source)
		return self._code

	def __str__(self):
		return self.code

	def __repr__(self):
		return f"Comment({repr(self.source)}, {self.code_span}, {self.text_span}, {self.multiline})"

	def __eq__(self, other):
		if isinstance(other, self.__class__):
			if self.__dict__ == other.__dict__:
				return True
		return False


class TestComment(unittest.TestCase):
	def test(self):

		c = Comment("Code /* commented */ like that", Span(5, 20), Span(7, 18), True)
		self.assertEqual(c.code, "/* commented */")
		self.assertEqual(c.text, " commented ")
		self.assertEqual(repr(c),
						 "Comment('Code /* commented */ like that', Span(start=5, end=20), Span(start=7, end=18), True)")
		self.assertEqual(str(c),
						 "/* commented */")




# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import List

from commie.x01_common import Comment
from .helper import minimize
from .. import iter_comments_c, UnterminatedCommentError


def commentsToList(code: str) -> List[Comment]:
	return list(iter_comments_c(code))


class CParserJsTest(unittest.TestCase):

	def testEmptyString(self):
		code = ""
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testSingleLineComment(self):
		code = '// single line comment'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "// single line comment")
		self.assertEqual(comments[0].text, " single line comment")
		self.assertEqual(comments[0].multiline, False)

	def testLineCommentInSingleQuotedLiteral(self):
		code = "msg = '// this is not a comment'"
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testLineCommentInDoubleQuotedLiteral(self):
		code = 'msg = "// this is not a comment"'
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testMultiLineComment(self):
		code = '/* multiline\ncomment */'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)
		self.assertEqual(comments[0].code, '/* multiline\ncomment */')
		self.assertEqual(comments[0].text, " multiline\ncomment ")
		self.assertEqual(comments[0].multiline, True)

	def testMultiLineCommentWithStars(self):
		code = "/***************/"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)
		self.assertEqual(comments[0].code, '/***************/')
		self.assertEqual(comments[0].text, "*************")
		self.assertEqual(comments[0].multiline, True)

	def testMultiLineCommentInSingleLiteral(self):
		code = "msg = '/* This is not a\\nmultiline comment */'"
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testMultiLineCommentInDoubleLiteral(self):
		code = 'msg = "/* This is not a\\nmultiline comment */"'
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testMultiLineCommentUnterminated(self):
		code = 'a = 1 /* Unterminated\\n comment'
		with self.assertRaises(UnterminatedCommentError):
			commentsToList(code)

	def testFragment(self):
		code = """
		
		/**
		 * JSDoc is used to annotate JavaScript.
		 *
		 * @author: Wikipedia
		 */
		 
		// single line comment
	
		let forgetIt = prompt("What?");
		
		// comment at eof"""

		comments = commentsToList(code)

		self.assertEqual(len(comments), 3)

		self.assertEqual(minimize(comments[0].code),
						 '/** * JSDoc is used to annotate JavaScript. * * @author: Wikipedia */')
		self.assertEqual(minimize(comments[0].text),
						 '* * JSDoc is used to annotate JavaScript. * * @author: Wikipedia')
		self.assertEqual(comments[0].multiline, True)

		self.assertEqual(comments[1].code, '// single line comment')
		self.assertEqual(comments[1].text, ' single line comment')
		self.assertEqual(comments[1].multiline, False)


		self.assertEqual(comments[2].code, '// comment at eof')
		self.assertEqual(comments[2].text, ' comment at eof')
		self.assertEqual(comments[2].multiline, False)



# SPDX-FileCopyrightText: Copyright (c) 2021 Art Galkin <ortemeo.werhal.com>
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import *

from .x01_common import Comment


def _startsTheLine(text: str, pos: int) -> bool:
	"""Returns True if the line contains only blanks to
	the left to the pos"""

	prevNewLinePos = text.rfind("\n", 0, pos + 1)
	# if this is the first string, we'll get prevNewLinePos=-1.
	# But we still can get the followining substring:
	lineStart = text[prevNewLinePos + 1:pos]
	return not lineStart.strip()  # true if all blanks


class TestStartsTheLine(unittest.TestCase):

	def test_first_string(self):
		text = "   abc"
		self.assertEqual(_startsTheLine(text, text.find("a")), True)
		self.assertEqual(_startsTheLine(text, text.find("b")), False)

	def test_multi_string(self):
		text = """	abc
					12345
					def"""

		self.assertEqual(len(text.splitlines()), 3)

		self.assertEqual(_startsTheLine(text, text.find("a")), True)
		self.assertEqual(_startsTheLine(text, text.find("b")), False)
		self.assertEqual(_startsTheLine(text, text.find("c")), False)

		self.assertEqual(_startsTheLine(text, text.find("1")), True)
		self.assertEqual(_startsTheLine(text, text.find("2")), False)
		self.assertEqual(_startsTheLine(text, text.find("3")), False)
		self.assertEqual(_startsTheLine(text, text.find("4")), False)
		self.assertEqual(_startsTheLine(text, text.find("5")), False)

		self.assertEqual(_startsTheLine(text, text.find("d")), True)
		self.assertEqual(_startsTheLine(text, text.find("e")), False)
		self.assertEqual(_startsTheLine(text, text.find("f")), False)


def _oneEmptyLineBetween(text: str, start: int, end: int) -> bool:
	between = text[start:end]
	linesBetween = between.splitlines()
	return len(linesBetween) == 2 and not any(s.strip() for s in linesBetween)


def group_singleline_comments(comments: Iterable[Comment]) -> Iterable[List[Comment]]:
	"""Combines adjacent single-line comments into groups."""

	group: List[Comment] = []

	for comment in comments:

		if not comment.multiline and _startsTheLine(comment.source, comment.code_span.start):
			if group and not _oneEmptyLineBetween(comment.source, group[-1].code_span.end,
												  comment.code_span.start):
				if group:
					yield group
					group = []
			group.append(comment)
			continue

		if group:
			yield group
			group = []
		yield [comment]

	if group:
		yield group


class TestGlue(unittest.TestCase):

	def testEmpty(self):
		from .parsers import iter_comments_c
		source = ""
		groups = list(group_singleline_comments(iter_comments_c(source)))
		self.assertEqual(len(groups), 0)


	def testThreeSingleLines(self):
		from .parsers import iter_comments_c

		source = """
			// three single line
			// comments will be treated
			// as a single comment
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))

		self.assertEqual(len(groups), 1)
		self.assertEqual(len(groups[0]), 3)

	def testThreeSingleLinesTooMuchSpaces(self):
		from .parsers import iter_comments_c

		source = """
			// comment a
			
			// comment b
			
			
			// comment c
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))
		self.assertEqual(len(groups), 3)

	def testThreeMultiLines(self):
		from .parsers import iter_comments_c

		source = """
			/* comment a */
			/* comment b */
			/* comment c */
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))
		self.assertEqual(len(groups), 3)

	def testSSM(self):
		from .parsers import iter_comments_c

		source = """
			// comment a
			// comment b
			/* comment c */
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))
		self.assertEqual(len(groups), 2)
		self.assertEqual(len(groups[0]), 2)
		self.assertEqual(len(groups[1]), 1)

	def testMSS(self):
		from .parsers import iter_comments_c

		source = """
			/* comment a */
			// comment b
			// comment c
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))
		self.assertEqual(len(groups), 2)
		self.assertEqual(len(groups[0]), 1)
		self.assertEqual(len(groups[1]), 2)

	def testSMS(self):
		from .parsers import iter_comments_c

		source = """
			// comment a
			/* comment b */
			// comment c
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))
		self.assertEqual(len(groups), 3)

	def testMSM(self):
		from .parsers import iter_comments_c

		source = """
			/* comment a */
			// comment b
			/* comment c */
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))
		self.assertEqual(len(groups), 3)

	def testMix1(self):
		from .parsers import iter_comments_c

		source = """
			// first single comment
			#include<stdio.h>

			// three single line
			// comments will be treated
			// as a single comment

			void main() {
				// one more single comment
				print("bye!");
			}
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))

		for g in groups:
			print(len(g), g[0].text)

		self.assertEqual(len(groups), 3)
		self.assertEqual(len(groups[0]), 1)
		self.assertEqual(len(groups[1]), 3)
		self.assertEqual(len(groups[2]), 1)

	def testMix2(self):
		from .parsers import iter_comments_c

		source = """
			#include<stdio.h>

			// 0 three single line
			// 0 comments will be treated
			// 0 as a single comment
			
			// 1 single
			int x; // 2 one more single
			
			// 3 empty line above
			// 3 makes it easier

			void main() {
				print("bye!");
			}
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))

		self.assertEqual(len(groups), 4)
		self.assertEqual(len(groups[0]), 3)
		self.assertEqual(len(groups[1]), 1)
		self.assertEqual(len(groups[2]), 1)
		self.assertEqual(len(groups[3]), 2)

	def testMix3(self):
		from .parsers import iter_comments_c

		source = """
			#include<stdio.h>

			// 0 three single line
			// 0 comments will be treated
			// 0 as a single comment
			
			// 1 single
			int x; // 2 one more single
			// 3 no lines above
			// 3 but it's not glued to 2

			void main() {
				print("bye!");
			}
		"""

		groups = list(group_singleline_comments(iter_comments_c(source)))

		for g in groups:
			print(len(g))

		self.assertEqual(len(groups), 4)
		self.assertEqual(len(groups[0]), 3)
		self.assertEqual(len(groups[1]), 1)
		self.assertEqual(len(groups[2]), 1)
		self.assertEqual(len(groups[3]), 2)


if __name__ == "__main__":
	unittest.main()



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import List

from commie import iter_comments_shell, Comment


def commentsToList(code: str) -> List[Comment]:
	return list(iter_comments_shell(code))


class ShellParserTest(unittest.TestCase):

	def testEmptyString(self):
		code = ""
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testComment(self):
		code = '# comment'
		comments = commentsToList(code)
		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '# comment')
		self.assertEqual(comments[0].text, " comment")
		self.assertEqual(comments[0].multiline, False)

	def testEscapedComment(self):
		code = r'\# not a comment'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testCommentInSingleQuotedString(self):
		code = "'this is # not a comment'"
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testCommentInDoubleQuotedString(self):
		code = '"this is # not a comment"'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testNestedStringSingleOutside(self):
		code = "'this is \"# not a comment\"'"
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testNestedStringDoubleOutside(self):
		code = '"this is \'# not a comment\'"'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testEscapedSingleQuote(self):
		code = "\\'# this is a comment"
		comments = commentsToList(code)
		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "# this is a comment")
		self.assertEqual(comments[0].text, " this is a comment")
		self.assertEqual(comments[0].multiline, False)

	def testEscapedDoubleQuote(self):
		code = '\\"# this is another comment'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "# this is another comment")
		self.assertEqual(comments[0].text, " this is another comment")
		self.assertEqual(comments[0].multiline, False)



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

"""This module provides methods for parsing comments from HTML family languages.

Works with:
  HTML
  XML
  SGML
"""

import re
from typing import Iterable

import commie.x01_errors
from commie.parsers._helper import matchGroupToComment
from commie.x01_common import Comment


def extract_comments(htmlCode: str) -> Iterable[Comment]:
	"""Extracts a list of comments from the given HTML family source code.

	Comments are represented with the Comment class found in the common module.
	HTML family comments come in one form, comprising all text within '<!--' and
	'-->' markers. Comments cannot be nested.

	Args:
	  htmlCode: String containing code to extract comments from.
	Returns:
	  Python list of common.Comment in the order that they appear in the code..
	Raises:
	  common.UnterminatedCommentError: Encountered an unterminated multi-line
		comment.
	"""
	pattern = r"""
		(?P<literal> (\"([^\"\n])*\")+) |
		(?P<single> <!--(?P<single_content>.*?)-->) |
		(?P<multi> <!--(?P<multi_content>(.|\n)*?)?-->) |
		(?P<error> <!--(.*)?)
	  """
	compiled = re.compile(pattern, re.VERBOSE | re.MULTILINE)

	for match in compiled.finditer(htmlCode):

		kind = match.lastgroup

		if kind == "single":
			# all the comments in HTML are multi-line
			yield matchGroupToComment(match, "single_content", True)
		elif kind == "multi":
			yield matchGroupToComment(match, "multi_content", True)
		elif kind == "error":
			raise commie.x01_errors.UnterminatedCommentError()



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

from typing import Iterable

from commie.x01_common import Comment, Span


def extract_comments(code: str) -> Iterable[Comment]:
	"""Extracts a list of comments from the given shell script.
	Comments are represented with the Comment class found in the common module.
	Shell script comments only come in one form, single-line. Single line
	comments start with an unquoted or unescaped '#' and continue on until the
	end of the line. A quoted '#' is one that is located within a pair of
	matching single or double quote marks. An escaped '#' is one that is
	immediately preceeded by a backslash '\'
	Args:
	  code: String containing code to extract comments from.
	Returns:
	  Python list of common.Comment in the order that they appear in the code.
	"""

	DEFAULT = 0
	IN_COMMENT = 1
	IN_STRING = 2
	ESCAPING_CHAR_INSIDE_STRING = 3
	ESCAPING_CHAR_OUTSIDE_OF_STRING = 4

	state = DEFAULT
	string_char = ''
	current_comment_text = ''
	line_counter = 1

	comment_start_pos = None
	position = -1

	for position, char in enumerate(code):
		if state == DEFAULT:
			# Waiting for comment start character, beginning of string,
			# or escape character.
			if char == '#':
				state = IN_COMMENT
				comment_start_pos = position
			elif char in ('"', "'"):
				string_char = char
				state = IN_STRING
			elif char == '\\':
				state = ESCAPING_CHAR_OUTSIDE_OF_STRING
		elif state == IN_COMMENT:
			if char == '\n':
				yield Comment(code, code_span=Span(comment_start_pos, position),
							  text_span=Span(comment_start_pos + 1, position), multiline=False)
				current_comment_text = ''
				state = DEFAULT
			else:
				current_comment_text += char
		elif state == IN_STRING:
			if char == string_char:
				state = DEFAULT
			elif char == '\\':
				state = ESCAPING_CHAR_INSIDE_STRING
		elif state == ESCAPING_CHAR_INSIDE_STRING:
			state = IN_STRING
		elif state == ESCAPING_CHAR_OUTSIDE_OF_STRING:
			# Escaping current char, outside of string.
			state = DEFAULT
		if char == '\n':
			line_counter += 1

	# end of file

	if state == IN_COMMENT:
		yield Comment(code, code_span=Span(comment_start_pos, position + 1),
					  text_span=Span(comment_start_pos + 1, position + 1), multiline=False)



from .c_parser_regex import extract_comments as iter_comments_sass
from .c_parser_state import iter_comments_c, iter_comments_go
from .css_parser_regex import extract_comments as iter_comments_css
from .html_parser_regex import extract_comments as iter_comments_html
from .python_parser import extract_comments as iter_comments_python
from .ruby_parser_regex import extract_comments as iter_comments_ruby
from .shell_parser_state import extract_comments as iter_comments_shell



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import List

from commie import iter_comments_c, UnterminatedCommentError
from commie.tests.helper import minimize
from commie.x01_common import Comment


def commentsToList(code: str) -> List[Comment]:
	return list(iter_comments_c(code))
	#return list(iter_comments_c(code))


class CParserTest(unittest.TestCase):

	def testSimpleMain(self):
		code = "// this is a comment\nint main() {\nreturn 0;\n}\n"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "// this is a comment")
		self.assertEqual(comments[0].text, " this is a comment")
		self.assertEqual(comments[0].multiline, False)

	def testSingleLineComment(self):
		code = '// single line comment'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '// single line comment')
		self.assertEqual(comments[0].text, " single line comment")
		self.assertEqual(comments[0].multiline, False)

	def testSingleLineCommentInStringLiteral(self):
		code = 'char* msg = "// this is not a comment"'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testMultiLineComment(self):
		code = '/* multiline\ncomment */'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '/* multiline\ncomment */')
		self.assertEqual(comments[0].text, " multiline\ncomment ")
		self.assertEqual(comments[0].multiline, True)

	def testMultiLineCommentWithStars(self):
		code = "/***************/"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '/***************/')
		self.assertEqual(comments[0].text, "*************")
		self.assertEqual(comments[0].multiline, True)

	def testMultiLineCommentInStringLiteral(self):
		code = 'char* msg = "/* This is not a\\nmultiline comment */"'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testMultiLineCommentUnterminated(self):
		code = 'int a = 1; /* Unterminated\\n comment'
		with self.assertRaises(UnterminatedCommentError):
			commentsToList(code)

	def testMultipleMultilineComments(self):
		code = '/* abc */ /* 123 */'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 2)

		self.assertEqual(comments[0].code, '/* abc */')
		self.assertEqual(comments[0].text, " abc ")
		self.assertEqual(comments[0].multiline, True)

		self.assertEqual(comments[1].code, '/* 123 */')
		self.assertEqual(comments[1].text, " 123 ")
		self.assertEqual(comments[1].multiline, True)

	def testStringThenComment(self):
		code = r'"" /* "abc */'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '/* "abc */')
		self.assertEqual(comments[0].text, ' "abc ')
		self.assertEqual(comments[0].multiline, True)

	def testCommentStartInsideEscapedQuotesInStringLiteral(self):
		#FIXME: This one fails with UnterminatedCommentError
		code = r'" \" /* \" "'
		comments = commentsToList(code)
		self.assertEqual(comments, [])
		pass

	def testStringEscapedBackslashCharacter(self):
		code = r'"\\"'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testTwoStringsFollowedByComment(self):
		code = r'"""" // foo'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '// foo')
		self.assertEqual(comments[0].text, " foo")
		self.assertEqual(comments[0].multiline, False)

	def testCommentedMultilineComment(self):
		code = '''// What if i start a /* here
    		int main(){return 0;}
    		// and ended it here */'''

		comments = commentsToList(code)

		self.assertEqual(len(comments), 2)

		self.assertEqual(comments[0].code, '// What if i start a /* here')
		self.assertEqual(comments[0].text, " What if i start a /* here")
		self.assertEqual(comments[0].multiline, False)

		self.assertEqual(comments[1].code, '// and ended it here */')
		self.assertEqual(comments[1].text, " and ended it here */")
		self.assertEqual(comments[1].multiline, False)

	def testMultilineCommentedComment(self):
		code = '''
			/*// here
			int main(){return 0;}
			*/// and ended it here */
		'''

		comments = commentsToList(code)

		self.assertEqual(len(comments), 2)

		self.assertEqual(minimize(comments[0].code), "/*// here int main(){return 0;} */")
		self.assertEqual(minimize(comments[0].text), "// here int main(){return 0;}")
		self.assertEqual(comments[0].multiline, True)

		self.assertEqual(comments[1].code, '// and ended it here */')
		self.assertEqual(comments[1].text, " and ended it here */")
		self.assertEqual(comments[1].multiline, False)

	def testEmptyString(self):
		code = ""
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)


	def testFragment(self):
		code = """
		
		/**
		 * JSDoc is rarely used to annotate C.
		 *
		 * @param: abc
		 */		
	
		int main() { return 42; } 
	
		// comment at eof"""

		comments = commentsToList(code)

		self.assertEqual(len(comments), 2)

		self.assertEqual(minimize(comments[0].code),
						 '/** * JSDoc is rarely used to annotate C. * * @param: abc */')
		self.assertEqual(minimize(comments[0].text),
						 '* * JSDoc is rarely used to annotate C. * * @param: abc')
		self.assertEqual(comments[0].multiline, True)

		self.assertEqual(comments[1].code, '// comment at eof')
		self.assertEqual(comments[1].text, ' comment at eof')
		self.assertEqual(comments[1].multiline, False)




# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from pathlib import Path
from typing import Iterable, Union

from commie.parsers import *
from commie.x01_common import Comment
from commie.x01_errors import *


def pickfunc(filename: str):
	filename = filename.lower()
	ext = filename.rpartition(".")[-1]

	if ext in ["c", "cpp", "java",
			   "h", "hpp",
			   # Objective-C source code 'implementation' program files usually
			   # have .m filename extensions, while Objective-C 'header/interface' files
			   # have .h extensions
			   "m"]:
		return iter_comments_c

	if ext in ["go"]:
		return iter_comments_go

	if ext in ["js", "ts", "dart"]:
		return iter_comments_c

	if ext in ["html", "htm", "xml"]:
		return iter_comments_html

	if ext in ["rb"]:
		return iter_comments_ruby

	if ext in ["py"]:
		return iter_comments_python

	if ext in ["scss"]:
		return iter_comments_sass

	if ext in ["css"]:
		return iter_comments_css

	if ext in ["sh"]:
		return iter_comments_shell

	raise FormatUndetectedError


class TestPickFunc(unittest.TestCase):
	def test_html(self):
		self.assertEqual(pickfunc(filename="file.html"), iter_comments_html)
		self.assertEqual(pickfunc(filename="1991.HTM"), iter_comments_html)

	def test_js(self):
		self.assertEqual(pickfunc(filename="file.dart"), iter_comments_c)
		self.assertEqual(pickfunc(filename="file.js"), iter_comments_c)

	def test_undetected(self):
		with self.assertRaises(FormatUndetectedError):
			pickfunc(filename="ladeda.haha")


def iter_comments_str(code: str, filename: str) -> Iterable[Comment]:
	func = pickfunc(filename)
	return func(code)


def iter_comments_file(file: Path) -> Iterable[Comment]:
	return iter_comments_str(file.read_text(), file.name)


def iter_comments(codeOrFile: Union[Path, str], filename: str = None) -> Iterable[Comment]:
	if isinstance(codeOrFile, str):
		if filename is None:
			raise ValueError("Please specify filename")
		return iter_comments_str(codeOrFile, filename)
	return iter_comments_file(codeOrFile)


class TestIterComments(unittest.TestCase):
	def testNoFilename(self):
		with self.assertRaises(ValueError):
			list(iter_comments("..."))



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

import re
from typing import Iterable

from commie.parsers._helper import matchGroupToComment
from commie.x01_common import Comment


def extract_comments(rubyCode: str) -> Iterable[Comment]:
	"""Extracts a list of comments from the given Ruby source code.

	Comments are represented with the Comment class found in the common module.

	Ruby comments start with a '#' character and run to the end of the line,
	http://ruby-doc.com/docs/ProgrammingRuby.

	Args:
	  rubyCode: String containing code to extract comments from.
	Returns:
	  Python list of common.Comment in the order that they appear in the code..
	"""
	pattern = r"""
		(?P<literal> ([\"'])((?:\\\2|(?:(?!\2)).)*)(\2)) |
		(?P<single> \#(?P<single_content>.*?)$)
	  """
	compiled = re.compile(pattern, re.VERBOSE | re.MULTILINE)

	#comments: List = []
	for match in compiled.finditer(rubyCode):
		kind = match.lastgroup

		if kind == "single":
			yield matchGroupToComment(match, "single_content", False)

	#return comments



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

# AG 2021: This was the default C/C++/Java parser in pypi/comment_parser.
# But the JS parser was much better at this task.
#
# The parser will remain here for now for simpler things like SASS

import re
from typing import Iterable

import commie.x01_errors
from commie.parsers._helper import matchGroupToComment
from commie.x01_common import Comment


def extract_comments(code: str) -> Iterable[Comment]:
	"""Extracts a list of comments from the given C family source code.

	Comments are represented with the Comment class found in the common module.
	C family comments come in two forms, single and multi-line comments.
	  - Single-line comments begin with '//' and continue to the end of line.
	  - Multi-line comments begin with '/*' and end with '*/' and can span
		multiple lines of code. If a multi-line comment does not terminate
		before EOF is reached, then an exception is raised.

	!!! Note that this doesn't take language-specific preprocessor directives into
	consideration.

	"""
	pattern = r"""
		(?P<literal> (\"([^\"\n])*\")+) |
		(?P<single> //(?P<single_content>.*)?$) |
		(?P<multi> /\*(?P<multi_content>(.|\n)*?)?\*/) |
		(?P<error> /\*(.*)?)
	  """

	compiled = re.compile(pattern, re.VERBOSE | re.MULTILINE)

	for match in compiled.finditer(code):

		kind = match.lastgroup
		# markupSpan = match.span(0)

		if kind == "single":
			yield matchGroupToComment(match, "single_content", False)

		elif kind == "multi":
			yield matchGroupToComment(match, "multi_content", True)

		elif kind == "error":
			raise commie.x01_errors.UnterminatedCommentError()



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

class Error(Exception):
	"""Base Error class for all comment parsers."""


class FileError(Error):
	"""Raised if there is an issue reading a file."""


class UnterminatedCommentError(Error):
	"""Raised if an Unterminated multi-line comment is encountered."""


class FormatUndetectedError(Error):
	"""Raised if there is an issue reading a file."""



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import List

from commie.x01_common import Comment
from .. import iter_comments_html, UnterminatedCommentError


def commentsToList(code: str) -> List[Comment]:
	return list(iter_comments_html(code))


class HtmlParserTest(unittest.TestCase):

	def testEmptyString(self):
		code = ""
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testComment(self):
		code = '<!--comment-->'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "<!--comment-->")
		self.assertEqual(comments[0].text, "comment")
		self.assertEqual(comments[0].multiline, True)

	def testMultilineComment(self):
		code = '<!--multi-line\ncomment-->'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '<!--multi-line\ncomment-->')
		self.assertEqual(comments[0].text, "multi-line\ncomment")
		self.assertEqual(comments[0].multiline, True)

	def testTwoSeparateSingleComment(self):
		code = '<!--comment1-->\n<!--comment2-->'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 2)

		self.assertEqual(comments[0].code, '<!--comment1-->')
		self.assertEqual(comments[0].text, "comment1")
		self.assertEqual(comments[0].multiline, True)

		self.assertEqual(comments[1].code, '<!--comment2-->')
		self.assertEqual(comments[1].text, "comment2")
		self.assertEqual(comments[1].multiline, True)

	def testLayeredComment(self):
		code = '<!-- comment<!-- -->'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '<!-- comment<!-- -->')
		self.assertEqual(comments[0].text, " comment<!-- ")
		self.assertEqual(comments[0].multiline, True)

	def testNonGreedyComment(self):
		code = '<!--i am a comment--> not a comment -->'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '<!--i am a comment-->')
		self.assertEqual(comments[0].text, "i am a comment")
		self.assertEqual(comments[0].multiline, True)

	def testSideBySideComment(self):
		code = '<!--comment1--> ... <!--comment2-->'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 2)

		self.assertEqual(comments[0].code, '<!--comment1-->')
		self.assertEqual(comments[0].text, "comment1")
		self.assertEqual(comments[0].multiline, True)

		self.assertEqual(comments[1].code, '<!--comment2-->')
		self.assertEqual(comments[1].text, "comment2")
		self.assertEqual(comments[1].multiline, True)

	def testUnterminatedComment(self):
		code = '<!--invalid'
		with self.assertRaises(UnterminatedCommentError):
			commentsToList(code)

	def testLonelyTerminator(self):
		code = 'not a comment-->'
		comments = commentsToList(code)
		self.assertEqual(comments, [])



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import List

from commie import iter_comments_css
from commie.x01_common import Comment


def commentsToList(code: str) -> List[Comment]:
	return list(iter_comments_css(code))


class CssParserTest(unittest.TestCase):

	def testEmptyString(self):
		code = ""
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testSimpleMain(self):
		code = ".cssClass { /* i am\n a comment! */ }"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "/* i am\n a comment! */")
		self.assertEqual(comments[0].text, " i am\n a comment! ")
		self.assertEqual(comments[0].multiline, True)

	def testIncLeft(self):
		code = ".cssClass { /* /* i am a comment! */ }"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "/* /* i am a comment! */")
		self.assertEqual(comments[0].text, " /* i am a comment! ")
		self.assertEqual(comments[0].multiline, True)

	def testIncRight(self):
		code = ".cssClass { /* i am a comment! */ */ }"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "/* i am a comment! */")
		self.assertEqual(comments[0].text, " i am a comment! ")
		self.assertEqual(comments[0].multiline, True)

	def testThreeComments(self):
		code = """
		/* THIS IS A CSS CODE
		   WITH MULTILINE COMMENTS */
		
		body {margin: 0; /* comment inside */}
		p {margin: 1px; ) /* comment outside */
		
		"""
		comments = commentsToList(code)

		self.assertEqual(len(comments), 3)



# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: BSD-3-Clause

import unittest
from typing import List

from commie import iter_comments_go, Comment, UnterminatedCommentError


def commentsToList(code: str) -> List[Comment]:
	return list(iter_comments_go(code))


class GoParserTest(unittest.TestCase):

	def testEmptyString(self):
		code = ""
		comments = commentsToList(code)
		self.assertEqual(len(comments), 0)

	def testSingleLineComment(self):

		code = '// single line comment'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "// single line comment")
		self.assertEqual(comments[0].text, " single line comment")
		self.assertEqual(comments[0].multiline, False)

	def testSingleLineCommentInRuneLiteral(self):
		code = "msg := '// this is not a comment'"
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testSingleLineCommentInBackTickedLiteral(self):
		code = "msg := `// this is not a comment`"
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testSingleLineCommentInQuotedLiteral(self):
		code = 'msg := "// this is not a comment"'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testMultiLineComment(self):
		code = '/* multiline\ncomment */'
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, '/* multiline\ncomment */')
		self.assertEqual(comments[0].text, " multiline\ncomment ")
		self.assertEqual(comments[0].multiline, True)

	def testMultiLineCommentWithStars(self):
		code = "/***************/"
		comments = commentsToList(code)

		self.assertEqual(len(comments), 1)

		self.assertEqual(comments[0].code, "/***************/")
		self.assertEqual(comments[0].text, "*************")
		self.assertEqual(comments[0].multiline, True)

	def testMultiLineCommentInRuneLiteral(self):
		code = "msg := '/* This is not a\\nmultiline comment */'"
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testMultiLineCommentInQuotedLiteral(self):
		code = 'msg := "/* This is not a\\nmultiline comment */"'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testMultiLineCommentInBackTickedLiteral(self):
		code = 'msg := `/* This is not a\\nmultiline comment */`'
		comments = commentsToList(code)
		self.assertEqual(comments, [])

	def testMultiLineCommentUnterminated(self):
		code = 'a := 1 /* Unterminated\\n comment'
		with self.assertRaises(UnterminatedCommentError):
			commentsToList(code)



from pathlib import Path

from setuptools import setup, find_packages

readme = (Path(__file__).parent / 'README.md').read_text()
readme = "# "+readme.partition("\n#")[-1]

setup(
  name="commie",
  version="1.0.6",

  author="Artёm IG",
  author_email="ortemeo@gmail.com",
  url='https://github.com/rtmigo/commie_py',

  packages=find_packages(),
  install_requires=[],

  description="Extracts comments from source code in different "
              "programming languages",

  long_description=readme,
  long_description_content_type='text/markdown',

  license='BSD-3-Clause',

  keywords="""
    css python c search java go html sass c-sharp dart bash parser 
    typescript parsing xml comments source-code""".split(),

  # https://pypi.org/classifiers/
  classifiers=[
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    'License :: OSI Approved :: BSD License',
    'Topic :: Software Development :: Documentation',
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Operating System :: OS Independent",
  ],
)


# SPDX-FileCopyrightText: Copyright (c) 2021 Artёm IG <github.com/rtmigo>
# SPDX-License-Identifier: BSD-3-Clause

import re
from typing import Iterable

import commie.x01_errors
from commie.parsers._helper import matchGroupToComment
from commie.x01_common import Comment


def extract_comments(cssCode: str) -> Iterable[Comment]:
	pattern = r"""
    (?P<comment> /\*(?P<content>(.|\n)*?)?\*/) |
    (?P<error> /\*(.*)?)
  """

	for match in re.finditer(pattern, cssCode, flags=re.VERBOSE | re.MULTILINE):

		kind = match.lastgroup

		if kind == "comment":
			yield matchGroupToComment(match, "content", True)

		elif kind == "error":
			raise commie.x01_errors.UnterminatedCommentError()
